@@ -64,10 +64,17 @@ class Firewall:
         self.zone = FirewallZone(self)
         self.direct = FirewallDirect(self)
         self.config = FirewallConfig(self)
-        self.policies = FirewallPolicies(self)
+        self.policies = FirewallPolicies()
 
         self.__init_vars()
 
+    def __repr__(self):
+        return '%s(%r, %r, %r, %r, %r, %r, %r, %r, %r, %r, %r)' % \
+            (self.__class__, self.ip4tables_enabled, self.ip6tables_enabled,
+             self.ebtables_enabled, self._state, self._panic,
+             self._default_zone, self._module_refcount, self._marks,
+             self._min_mark, self.cleanup_on_exit, self.ipv6_rpfilter_enabled)
+
     def __init_vars(self):
         self._state = "INIT"
         self._panic = False
@@ -76,6 +83,7 @@ class Firewall:
         self._marks = [ ]
         self._min_mark = FALLBACK_MINIMAL_MARK # will be overloaded by firewalld.conf
         self.cleanup_on_exit = True
+        self.ipv6_rpfilter_enabled = True
 
     def _check_tables(self):
         # check if iptables, ip6tables and ebtables are usable, else disable
@@ -132,6 +140,18 @@ class Firewall:
                         # already enabled, this is probably reload
                         pass
 
+            if self._firewalld_conf.get("IPv6_rpfilter"):
+                value = self._firewalld_conf.get("IPv6_rpfilter")
+                if value != None:
+                    if value.lower() in [ "no", "false" ]:
+                        self.ipv6_rpfilter_enabled = False
+                    if value.lower() in [ "yes", "true" ]:
+                        self.ipv6_rpfilter_enabled = True
+            if self.ipv6_rpfilter_enabled:
+                log.debug1("IPv6 rpfilter is enabled")
+            else:
+                log.debug1("IPV6 rpfilter is disabled")
+
         self.config.set_firewalld_conf(copy.deepcopy(self._firewalld_conf))
 
         # apply default rules
@@ -179,23 +199,27 @@ class Firewall:
         if error:
             sys.exit(1)
 
+        # apply settings for loaded zones
+        self.zone.apply_zones()
+
         # load direct rules
-        log.debug1("Loading direct rules file '%s'" % FIREWALLD_DIRECT)
         obj = Direct(FIREWALLD_DIRECT)
-        try:
-            obj.read()
-        except Exception as msg:
-            log.debug1("Failed to load direct rules file '%s': %s",
-                      FIREWALLD_DIRECT, msg)
-        else:
-            self.direct.set_config((obj.get_all_chains(), obj.get_all_rules()))
-            for ipv, args in obj.get_all_passthroughs().items():
-                for arg in args:
-                    try:
-                        self.direct.passthrough(ipv, arg)
-                    except FirewallError as error:
-                        log.warning(str(error))
-            # TODO: copy obj into config interface
+        if os.path.exists(FIREWALLD_DIRECT):
+            log.debug1("Loading direct rules file '%s'" % FIREWALLD_DIRECT)
+            try:
+                obj.read()
+            except Exception as msg:
+                log.debug1("Failed to load direct rules file '%s': %s",
+                           FIREWALLD_DIRECT, msg)
+            else:
+                self.direct.set_config((obj.get_all_chains(),
+                                        obj.get_all_rules()))
+                for ipv, args in obj.get_all_passthroughs().items():
+                    for arg in args:
+                        try:
+                            self.direct.passthrough(ipv, arg)
+                        except FirewallError as error:
+                            log.warning(str(error))
         self.config.set_direct(copy.deepcopy(obj))
 
         # check if default_zone is a valid zone
@@ -275,23 +299,35 @@ class Firewall:
                     self.config.add_service(copy.deepcopy(obj))
                 elif reader_type == "zone":
                     obj = zone_reader(filename, path)
-                    if not combine:
-                        if obj.name in self.zone.get_zones():
-                            orig_obj = self.zone.get_zone(obj.name)
-                            if orig_obj.combined:
-                                raise FirewallError(NOT_OVERLOADABLE,
-                                                    "%s is a combined zone" % \
-                                                        obj.name)
+                    if combine:
+                        # Change name for permanent configuration
+                        obj.name = "%s/%s" % (
+                            os.path.basename(path),
+                            os.path.basename(filename)[0:-4])
+                        obj.check_name(obj.name)
+                    # Copy object before combine
+                    config_obj = copy.deepcopy(obj)
+                    if obj.name in self.zone.get_zones():
+                        orig_obj = self.zone.get_zone(obj.name)
+                        self.zone.remove_zone(orig_obj.name)
+                        if orig_obj.combined:
+                            log.debug1("  Combining %s '%s' ('%s/%s')",
+                                        reader_type, obj.name,
+                                        path, filename)
+                            obj.combine(orig_obj)
+                        else:
                             log.debug1("  Overloads %s '%s' ('%s/%s')",
                                        reader_type,
                                        orig_obj.name, orig_obj.path,
                                        orig_obj.filename)
-                            self.zone.remove_zone(orig_obj.name)
-                        self.zone.add_zone(obj)
-                        # add a deep copy to the configuration interface
-                        self.config.add_zone(copy.deepcopy(obj))
-                    else:
+                    self.config.add_zone(config_obj)
+                    if combine:
+                        log.debug1("  Combining %s '%s' ('%s/%s')",
+                                   reader_type, combined_zone.name,
+                                   path, filename)
                         combined_zone.combine(obj)
+                    else:
+                        self.zone.add_zone(obj)
                 else:
                     log.fatal("Unknown reader type %s", reader_type)
             except FirewallError as msg:
@@ -315,12 +351,14 @@ class Firewall:
             self.zone.add_zone(combined_zone)
 
     def cleanup(self):
-        self.__init_vars()
         self.icmptype.cleanup()
         self.service.cleanup()
         self.zone.cleanup()
         self.config.cleanup()
         self.direct.cleanup()
+        self.policies.cleanup()
+        self._firewalld_conf.cleanup()
+        self.__init_vars()
 
     def stop(self):
         if self.cleanup_on_exit:
@@ -482,6 +520,20 @@ class Firewall:
         for ipv in [ "ipv4", "ipv6", "eb" ]:
             self.__apply_default_rules(ipv)
 
+        if self.ipv6_rpfilter_enabled and \
+           self.is_table_available("ipv6", "raw"):
+            rule = [ "-t", "raw", "-I", "PREROUTING", "1",
+                     "-p", "icmpv6", "--icmpv6-type=router-advertisement",
+                     "-j", "ACCEPT" ]       # RHBZ#1058505
+            self.rule("ipv6", rule)
+            rule = [ "-t", "raw", "-I", "PREROUTING", "2",
+                     "-m", "rpfilter", "--invert", "-j", "DROP" ]
+            try:
+                self.rule("ipv6", rule)
+            except ValueError:    # some problem with ip6t_rpfilter module ?
+                rule = [ "-t", "raw", "-D", "PREROUTING", "1"]
+                self.rule("ipv6", rule)
+
     # flush and policy
 
     def _flush(self):
@@ -627,7 +679,7 @@ class Firewall:
         if _new_dz != _old_dz:
             # default zone changed. Move interfaces from old default zone to 
             # the new one.
-            for iface, settings in _zone_interfaces[_old_dz].items():
+            for iface, settings in list(_zone_interfaces[_old_dz].items()):
                 if settings["__default__"]:
                     # move only those that were added to default zone
                     # (not those that were added to specific zone same as 
@@ -710,7 +762,7 @@ class Firewall:
 
             # Move interfaces from old default zone to the new one.
             _old_dz_settings = self.zone.get_settings(_old_dz)
-            for iface, settings in _old_dz_settings["interfaces"].items():
+            for iface, settings in list(_old_dz_settings["interfaces"].items()):
                 if settings["__default__"]:
                     # move only those that were added to default zone
                     # (not those that were added to specific zone same as default)
