@@ -26,7 +26,7 @@ from firewall.functions import portStr, checkIPnMask, checkIP6nMask, \
     checkProtocol, enable_ip_forwarding, check_single_address
 from firewall.core.rich import *
 from firewall.errors import *
-from ipXtables import ip4tables_available_tables, ip6tables_available_tables
+from firewall.core.ipXtables import ip4tables_available_tables, ip6tables_available_tables
 
 mangle = []
 if "mangle" in ip4tables_available_tables:
@@ -73,14 +73,15 @@ INTERFACE_ZONE_OPTS = {
 class FirewallZone:
     def __init__(self, fw):
         self._fw = fw
-        self.__init_vars()
-
-    def __init_vars(self):
         self._chains = { }
         self._zones = { }
 
+    def __repr__(self):
+        return '%s(%r, %r)' % (self.__class__, self._chains, self._zones)
+
     def cleanup(self):
-        self.__init_vars()
+        self._chains.clear()
+        self._zones.clear()
 
     # zones
 
@@ -124,52 +125,53 @@ class FirewallZone:
 
         self._zones[obj.name] = obj
 
-        # load zone in case of missing services, icmptypes etc.
-        for args in obj.interfaces:
-            self._error2warning(self.add_interface, obj.name, args)
-        for args in obj.sources:
-            self._error2warning(self.add_source, obj.name, args)
-        for args in obj.icmp_blocks:
-            self._error2warning(self.add_icmp_block, obj.name, args)
-        for args in obj.forward_ports:
-            self._error2warning(self.add_forward_port, obj.name, *args)
-        for args in obj.services:
-            self._error2warning(self.add_service, obj.name, args)
-        for args in obj.ports:
-            self._error2warning(self.add_port, obj.name, *args)
-        if obj.masquerade:
-            self._error2warning(self.add_masquerade, obj.name)
-        for args in obj.rules:
-            self._error2warning(self.add_rule, obj.name, args)
-
     def remove_zone(self, zone):
         obj = self._zones[zone]
-        for args in obj.interfaces:
-            self._error2warning(self.remove_interface, obj.name, args)
-        for args in obj.sources:
-            self._error2warning(self.remove_source, obj.name, args)
-        for args in obj.icmp_blocks:
-            self._error2warning(self.remove_icmp_block, obj.name, args)
-        for args in obj.forward_ports:
-            self._error2warning(self.remove_forward_port, obj.name, *args)
-        for args in obj.services:
-            self._error2warning(self.remove_service, obj.name, args)
-        for args in obj.ports:
-            self._error2warning(self.remove_port, obj.name, *args)
-        if obj.masquerade:
-            self._error2warning(self.remove_masquerade, obj.name)
-        for args in obj.rules:
-            self._error2warning(self.remove_rule, obj.name, args)
-
+        if obj.applied:
+            self.unapply_zone_settings(zone)
         obj.settings.clear()
         del self._zones[zone]
 
+    def apply_zones(self):
+        for zone in self.get_zones():
+            obj = self._zones[zone]
+            applied = False
+
+            # load zone in case of missing services, icmptypes etc.
+            for args in obj.icmp_blocks:
+                self._error2warning(self.add_icmp_block, obj.name, args)
+            for args in obj.forward_ports:
+                self._error2warning(self.add_forward_port, obj.name, *args)
+            for args in obj.services:
+                self._error2warning(self.add_service, obj.name, args)
+            for args in obj.ports:
+                self._error2warning(self.add_port, obj.name, *args)
+            if obj.masquerade:
+                self._error2warning(self.add_masquerade, obj.name)
+            for args in obj.rules:
+                self._error2warning(self.add_rule, obj.name, args)
+            for args in obj.interfaces:
+                self._error2warning(self.add_interface, obj.name, args)
+                applied = True
+            for args in obj.sources:
+                self._error2warning(self.add_source, obj.name, args)
+                applied = True
+
+            obj.applied = applied
+
     # dynamic chain handling
 
     def __chain(self, zone, create, table, chain):
-        if zone in self._chains and table in self._chains[zone] and \
-                chain in self._chains[zone][table]:
-            return
+        if create:
+            if zone in self._chains and  \
+               table in self._chains[zone] and \
+               chain in self._chains[zone][table]:
+                return
+        else:
+            if zone not in self._chains or \
+               table not in self._chains[zone] or \
+               chain not in self._chains[zone][table]:
+                return
 
         chains = [ ]
         rules = [ ]
@@ -196,6 +198,18 @@ class FirewallZone:
                 rules.append((ipv, [ _zone, 3, "-t", table,
                                      "-j", "%s_allow" % (_zone) ]))
 
+                # Handle trust, block and drop zones:
+                # Add an additional rule with the zone target (accept, reject
+                # or drop) to the base _zone only in the filter table.
+                # Otherwise it is not be possible to have a zone with drop
+                # target, that is allowing traffic that is locally initiated
+                # or that adds additional rules. (RHBZ#1055190)
+                target = self._zones[zone].target
+                if table == "filter" and \
+                   target in [ "ACCEPT", "REJECT", "%%REJECT%%", "DROP" ] and \
+                   chain in [ "INPUT", "FORWARD_IN", "FORWARD_OUT", "OUTPUT" ]:
+                    rules.append((ipv, [ _zone, 4, "-t", table, "-j", target ]))
+
         if create:
             # handle chains first
             ret = self._fw.handle_chains(chains, create)
@@ -215,6 +229,8 @@ class FirewallZone:
                 self._fw.handle_rules(cleanup_rules, not create)
                 raise FirewallError(COMMAND_FAILED, msg)
         else:
+            # reverse rule order for cleanup
+            rules.reverse()
             # cleanup rules first
             ret = self._fw.handle_rules(rules, create, insert=True)
             if ret:
@@ -277,11 +293,7 @@ class FirewallZone:
                         # zone configuration, also do not restore date,
                         # sender and timeout
                         continue
-                    if key == "interfaces":
-                        self.change_zone_of_interface(zone, args)
-                    elif key == "sources":
-                        self.change_zone_of_source(zone, args)
-                    elif key == "icmp_blocks":
+                    if key == "icmp_blocks":
                         self.add_icmp_block(zone, args)
                     elif key == "forward_ports":
                         self.add_forward_port(zone, *args)
@@ -291,6 +303,12 @@ class FirewallZone:
                         self.add_port(zone, *args)
                     elif key == "masquerade":
                         self.add_masquerade(zone)
+                    elif key == "rules":
+                        self.add_rule(zone, Rich_Rule(rule_str=args))
+                    elif key == "interfaces":
+                        self.change_zone_of_interface(zone, args)
+                    elif key == "sources":
+                        self.change_zone_of_source(zone, args)
                     else:
                         log.error("Zone '%s': Unknown setting '%s:%s', "
                                   "unable to restore.", zone, key, args)
@@ -301,6 +319,51 @@ class FirewallZone:
         except FirewallError as msg:
             log.error(msg)
 
+    def __zone_settings(self, enable, zone):
+        obj = self.get_zone(zone)
+        if (enable and obj.applied) or (not enable and not obj.applied):
+            return
+        settings = self.get_settings(zone)
+        for key in settings:
+            for args in settings[key]:
+                try:
+                    if key == "icmp_blocks":
+                        self.__icmp_block(enable, zone, args)
+                    elif key == "forward_ports":
+                        mark = obj.settings["forward_ports"][args]["mark"]
+                        self.__forward_port(enable, zone, *args, mark_id=mark)
+                    elif key == "services":
+                        self.__service(enable, zone, args)
+                    elif key == "ports":
+                        self.__port(enable, zone, *args)
+                    elif key == "masquerade":
+                        self.__masquerade(enable, zone)
+                    elif key == "rules":
+                        mark = self.__rule(enable, zone,
+                                           Rich_Rule(rule_str=args), None)
+                        obj.settings["rules"][args]["mark"] = mark
+                    elif key == "interfaces":
+                        self.__interface(enable, zone, args)
+                    elif key == "sources":
+                        self.__source(enable, zone, args)
+                    else:
+                        log.error("Zone '%s': Unknown setting '%s:%s', "
+                                  "unable to apply", zone, key, args)
+                except FirewallError as msg:
+                    log.error(msg)
+        obj.applied = enable
+
+    def apply_zone_settings(self, zone):
+        self.__zone_settings(True, zone)
+
+    def unapply_zone_settings(self, zone):
+        self.__zone_settings(False, zone)
+
+    def unapply_zone_settings_if_unused(self, zone):
+        obj = self._zones[zone]
+        if len(obj.interfaces) == 0 and len(obj.sources) == 0:
+            self.unapply_zone_settings(zone)
+
     # handle chains, modules and rules for a zone
     def handle_cmr(self, zone, chains, modules, rules, enable):
         cleanup_chains = None
@@ -367,26 +430,20 @@ class FirewallZone:
         rules = [ ]
         for table in ZONE_CHAINS:
             for chain in ZONE_CHAINS[table]:
-                for ipv in ZONE_CHAINS[table][chain]:
-                    # create needed chains if not done already
-                    if enable:
-                        self.add_chain(zone, table, chain)
+                # create needed chains if not done already
+                if enable:
+                    self.add_chain(zone, table, chain)
 
-                    # handle trust and block zone directly, accept or reject
-                    # others will be placed into the proper zone chains
+                for ipv in ZONE_CHAINS[table][chain]:
+                    # handle all zones in the same way here, now
+                    # trust and block zone targets are handled now in __chain
                     opt = INTERFACE_ZONE_OPTS[chain]
-                    target = self._zones[zone].target.format(
+                    target = DEFAULT_ZONE_TARGET.format(
                         chain=SHORTCUTS[chain], zone=zone)
-                    if target in [ "REJECT", "%%REJECT%%" ] and \
-                            chain not in [ "INPUT", "FORWARD_IN", "FORWARD_OUT", "OUTPUT" ]:
-                        # REJECT is only valid in the INPUT, FORWARD and
-                        # OUTPUT chains, and user-defined chains which are 
-                        # only called from those chains
-                        continue
-                    if target == "DROP" and table == "nat":
-                        # DROP is not supported in nat table
-                        continue
-                    action = "-g" if "_ZONE_" in target else "-j"
+                    if self._zones[zone].target == DEFAULT_ZONE_TARGET:
+                        action = "-g"
+                    else:
+                        action = "-j"
                     rule = [ "%s_ZONES" % chain, "-t", table,
                              opt, interface, action, target ]
                     if enable and not append:
@@ -401,13 +458,17 @@ class FirewallZone:
             log.debug2(msg)
             raise FirewallError(COMMAND_FAILED, msg)
 
-        if not enable:
-            self.remove_chain(zone, table, chain)
+#        if not enable:
+#            for table in ZONE_CHAINS:
+#                for chain in ZONE_CHAINS[table]:
+#                    self.remove_chain(zone, table, chain)
 
     def add_interface(self, zone, interface, sender=None):
         self._fw.check_panic()
         _zone = self._fw.check_zone(zone)
         _obj = self._zones[_zone]
+        if not _obj.applied:
+            self.apply_zone_settings(zone)
 
         interface_id = self.__interface_id(interface)
 
@@ -441,6 +502,7 @@ class FirewallZone:
     def change_default_zone(self, old_zone, new_zone):
         self._fw.check_panic()
 
+        self.apply_zone_settings(new_zone)
         self.__interface(True, new_zone, "+", True)
         if old_zone != None and old_zone != "":
             self.__interface(False, old_zone, "+", True)
@@ -456,11 +518,13 @@ class FirewallZone:
 
         _obj = self._zones[_zone]
         interface_id = self.__interface_id(interface)
-        self.__interface(False, _zone, interface)
+        if _obj.applied:
+            self.__interface(False, _zone, interface)
 
         if interface_id in _obj.settings["interfaces"]:
             del _obj.settings["interfaces"][interface_id]
 
+#        self.unapply_zone_settings_if_unused(_zone)
         return _zone
 
     def query_interface(self, zone, interface):
@@ -514,7 +578,10 @@ class FirewallZone:
                     # DROP is not supported in nat table
                     continue
                 # append rule
-                action = "-g" if "_ZONE_" in target else "-j"
+                if self._zones[zone].target == DEFAULT_ZONE_TARGET:
+                    action = "-g"
+                else:
+                    action = "-j"
                 rule = [ "%s_ZONES_SOURCE" % chain, "-t", table,
                          opt, source, action, target ]
                 rules.append((ipv, rule))
@@ -527,13 +594,18 @@ class FirewallZone:
             log.debug2(msg)
             raise FirewallError(COMMAND_FAILED, msg)
 
-        if not enable:
-            self.remove_chain(zone, table, chain)
+#        if not enable:
+#            for table in ZONE_CHAINS:
+#                for chain in ZONE_CHAINS[table]:
+#                    self.remove_chain(zone, table, chain)
 
     def add_source(self, zone, source, sender=None):
         self._fw.check_panic()
         _zone = self._fw.check_zone(zone)
         _obj = self._zones[_zone]
+        if not _obj.applied:
+            self.apply_zone_settings(zone)
+
         source_id = self.__source_id(source)
 
         if source_id in _obj.settings["sources"]:
@@ -574,11 +646,13 @@ class FirewallZone:
 
         _obj = self._zones[_zone]
         source_id = self.__source_id(source)
-        self.__source(False, _zone, source_id[0], source_id[1])
+        if _obj.applied:
+            self.__source(False, _zone, source_id[0], source_id[1])
 
         if source_id in _obj.settings["sources"]:
             del _obj.settings["sources"][source_id]
 
+#        self.unapply_zone_settings_if_unused(_zone)
         return _zone
 
     def query_source(self, zone, source):
@@ -665,7 +739,6 @@ class FirewallZone:
         rules.append((ipv, table, chain, _command))
 
     def __rule(self, enable, zone, rule, mark_id):
-                       
         chains = [ ]
         modules = [ ]
         rules = [ ]
@@ -732,8 +805,8 @@ class FirewallZone:
 
                 table = "filter"
                 chains.append([ table, "INPUT" ])
-                target = self._zones[zone].target.format(chain=SHORTCUTS["INPUT"],
-                                                         zone=zone)
+                target = DEFAULT_ZONE_TARGET.format(chain=SHORTCUTS["INPUT"],
+                                                    zone=zone)
 
                 command = [ ]
                 self.__rule_source(rule.source, command)
@@ -753,8 +826,8 @@ class FirewallZone:
 
                 table = "filter"
                 chains.append([ table, "INPUT" ])
-                target = self._zones[zone].target.format(chain=SHORTCUTS["INPUT"],
-                                                         zone=zone)
+                target = DEFAULT_ZONE_TARGET.format(chain=SHORTCUTS["INPUT"],
+                                                    zone=zone)
 
                 command = [ ]
                 self.__rule_source(rule.source, command)
@@ -849,6 +922,7 @@ class FirewallZone:
 
                 if not enable:
                     self._fw.del_mark(mark_id)
+                    mark_id = None
 
             # ICMP BLOCK
             elif type(rule.element) == Rich_IcmpBlock:
@@ -937,7 +1011,10 @@ class FirewallZone:
         if rule_id in _obj.settings["rules"]:
             raise FirewallError(ALREADY_ENABLED)
 
-        mark = self.__rule(True, _zone, rule, None)
+        if _obj.applied:
+            mark = self.__rule(True, _zone, rule, None)
+        else:
+            mark = None
 
         _obj.settings["rules"][rule_id] = \
             self.__gen_settings(timeout, sender, mark=mark)
@@ -957,7 +1034,8 @@ class FirewallZone:
             mark = _obj.settings["rules"][rule_id]["mark"]
         else:
             mark = None
-        self.__rule(False, _zone, rule, mark)
+        if _obj.applied:
+            self.__rule(False, _zone, rule, mark)
 
         if rule_id in _obj.settings["rules"]:
             del _obj.settings["rules"][rule_id]
@@ -1044,7 +1122,8 @@ class FirewallZone:
         if service_id in _obj.settings["services"]:
             raise FirewallError(ALREADY_ENABLED)
 
-        self.__service(True, _zone, service)
+        if _obj.applied:
+            self.__service(True, _zone, service)
 
         _obj.settings["services"][service_id] = \
             self.__gen_settings(timeout, sender)
@@ -1060,7 +1139,8 @@ class FirewallZone:
         if not service_id in _obj.settings["services"]:
             raise FirewallError(NOT_ENABLED)
 
-        self.__service(False, _zone, service)
+        if _obj.applied:
+            self.__service(False, _zone, service)
 
         if service_id in _obj.settings["services"]:
             del _obj.settings["services"][service_id]
@@ -1117,7 +1197,8 @@ class FirewallZone:
         if port_id in _obj.settings["ports"]:
             raise FirewallError(ALREADY_ENABLED)
 
-        self.__port(True, _zone, port, protocol)
+        if _obj.applied:
+            self.__port(True, _zone, port, protocol)
 
         _obj.settings["ports"][port_id] = \
             self.__gen_settings(timeout, sender)
@@ -1133,7 +1214,8 @@ class FirewallZone:
         if not port_id in _obj.settings["ports"]:
             raise FirewallError(NOT_ENABLED)
 
-        self.__port(False, _zone, port, protocol)
+        if _obj.applied:
+            self.__port(False, _zone, port, protocol)
 
         if port_id in _obj.settings["ports"]:
             del _obj.settings["ports"][port_id]
@@ -1195,7 +1277,8 @@ class FirewallZone:
         if masquerade_id in _obj.settings["masquerade"]:
             raise FirewallError(ALREADY_ENABLED)
 
-        self.__masquerade(True, _zone)
+        if _obj.applied:
+            self.__masquerade(True, _zone)
 
         _obj.settings["masquerade"][masquerade_id] = \
             self.__gen_settings(timeout, sender)
@@ -1211,7 +1294,8 @@ class FirewallZone:
         if masquerade_id not in _obj.settings["masquerade"]:
             raise FirewallError(NOT_ENABLED)
 
-        self.__masquerade(False, _zone)
+        if _obj.applied:
+            self.__masquerade(False, _zone)
 
         if masquerade_id in _obj.settings["masquerade"]:
             del _obj.settings["masquerade"][masquerade_id]
@@ -1229,7 +1313,8 @@ class FirewallZone:
         if toport:
             self._fw.check_port(toport)
         if toaddr:
-            check_single_address(ipv, toaddr)
+            if not check_single_address(ipv, toaddr):
+                raise FirewallError(INVALID_ADDR, toaddr)
         if not toport and not toaddr:
             raise FirewallError(INVALID_FORWARD)
 
@@ -1309,8 +1394,9 @@ class FirewallZone:
             raise FirewallError(ALREADY_ENABLED)
 
         mark = self._fw.new_mark()
-        self.__forward_port(True, _zone, port, protocol, toport, toaddr,
-                            mark_id=mark)
+        if _obj.applied:
+            self.__forward_port(True, _zone, port, protocol, toport, toaddr,
+                                mark_id=mark)
         
         _obj.settings["forward_ports"][forward_id] = \
             self.__gen_settings(timeout, sender, mark=mark)
@@ -1329,8 +1415,9 @@ class FirewallZone:
 
         mark = _obj.settings["forward_ports"][forward_id]["mark"]
 
-        self.__forward_port(False, _zone, port, protocol, toport, toaddr,
-                            mark_id=mark)
+        if _obj.applied:
+            self.__forward_port(False, _zone, port, protocol, toport, toaddr,
+                                mark_id=mark)
 
         if forward_id in _obj.settings["forward_ports"]:
             del _obj.settings["forward_ports"][forward_id]
@@ -1405,7 +1492,8 @@ class FirewallZone:
         if icmp_id in _obj.settings["icmp_blocks"]:
             raise FirewallError(ALREADY_ENABLED)
 
-        self.__icmp_block(True, _zone, icmp)
+        if _obj.applied:
+            self.__icmp_block(True, _zone, icmp)
 
         _obj.settings["icmp_blocks"][icmp_id] = \
             self.__gen_settings(timeout, sender)
@@ -1421,7 +1509,8 @@ class FirewallZone:
         if not icmp_id in _obj.settings["icmp_blocks"]:
             raise FirewallError(NOT_ENABLED)
 
-        self.__icmp_block(False, _zone, icmp)
+        if _obj.applied:
+            self.__icmp_block(False, _zone, icmp)
 
         if icmp_id in _obj.settings["icmp_blocks"]:
             del _obj.settings["icmp_blocks"][icmp_id]
