@@ -21,6 +21,7 @@
 
 import xml.sax as sax
 import os
+import io
 import shutil
 
 from firewall.fw_types import *
@@ -35,6 +36,7 @@ class direct_ContentHandler(IO_Object_ContentHandler):
         self.direct = False
 
     def startElement(self, name, attrs):
+        IO_Object_ContentHandler.startElement(self, name)
         self.item.parser_check_element_attrs(name, attrs)
 
         if name == "direct":
@@ -46,22 +48,22 @@ class direct_ContentHandler(IO_Object_ContentHandler):
             if not self.direct:
                 log.error("Parse Error: chain outside of direct")
                 return
-            ipv = str(attrs["ipv"])
-            table = str(attrs["table"])
-            chain = str(attrs["chain"])
+            ipv = attrs["ipv"]
+            table = attrs["table"]
+            chain = attrs["chain"]
             self.item.add_chain(ipv, table, chain)
 
         elif name == "rule":
             if not self.direct:
                 log.error("Parse Error: rule outside of direct")
                 return
-            ipv = str(attrs["ipv"])
+            ipv = attrs["ipv"]
             if ipv not in [ "ipv4", "ipv6", "eb" ]:
                 raise FirewallError(INVALID_IPV, ipv)
-            table = str(attrs["table"])
-            chain = str(attrs["chain"])
+            table = attrs["table"]
+            chain = attrs["chain"]
             try:
-                priority = int(str(attrs["priority"]))
+                priority = int(attrs["priority"])
             except:
                 log.error("Parse Error: %s is not a valid priority" % 
                           attrs["priority"])
@@ -72,7 +74,7 @@ class direct_ContentHandler(IO_Object_ContentHandler):
             if not self.direct:
                 log.error("Parse Error: command outside of direct")
                 return
-            ipv = str(attrs["ipv"])
+            ipv = attrs["ipv"]
             self._passthrough = [ipv]
 
         else:
@@ -105,7 +107,7 @@ class Direct(IO_Object):
 
     IMPORT_EXPORT_STRUCTURE = (
         # chain: [ ipv, table, [ chain ] ]
-        ( "chains", [ ( "", "", "" ), ], ),                   # a(ssas)
+        ( "chains", [ ( "", "", "" ), ], ),                   # a(sss)
         # rule: [ ipv, table, chain, [ priority, [ arg ] ] ]
         ( "rules", [ ( "", "", "", 0, [ "" ] ), ], ),         # a(sssias)
         # passthrough: [ ipv, [ [ arg ] ] ]
@@ -124,7 +126,9 @@ class Direct(IO_Object):
     def __init__(self, filename):
         super(Direct, self).__init__()
         self.filename = filename
-        self.clear()
+        self.chains = LastUpdatedOrderedDict()
+        self.rules = LastUpdatedOrderedDict()
+        self.passthroughs = LastUpdatedOrderedDict()
 
     def _check_config(self, config, item):
         pass
@@ -150,7 +154,7 @@ class Direct(IO_Object):
         return tuple(ret)
 
     def import_config(self, config):
-        self.clear()
+        self.cleanup()
         self.check_config(config)
         for i,(element,value) in enumerate(self.IMPORT_EXPORT_STRUCTURE):
             if element == "chains":
@@ -163,10 +167,10 @@ class Direct(IO_Object):
                 for x in config[i]:
                     self.add_passthrough(*x)
 
-    def clear(self):
-        self.chains = LastUpdatedOrderedDict()
-        self.rules = LastUpdatedOrderedDict()
-        self.passthroughs = LastUpdatedOrderedDict()
+    def cleanup(self):
+        self.chains.clear()
+        self.rules.clear()
+        self.passthroughs.clear()
 
     def output(self):
         print ("chains")
@@ -196,28 +200,28 @@ class Direct(IO_Object):
                             (chain, table, ipv)
                         + "already in list, ignoring")
 
-#    def remove_chain(self, ipv, table, chain):
-#        key = (ipv, table)
-#        if key in self.chains and chain in self.chains[key]:
-#            self.chains[key].remove(chain)
-#            if len(self.chains[key]) == 0:
-#                del self.chains[key]
-#        else:
-#            raise ValueError( \
-#                "Chain '%s' with table '%s' with ipv '%s' not in list" % \
-#                (chain, table, ipv))
+    def remove_chain(self, ipv, table, chain):
+        key = (ipv, table)
+        if key in self.chains and chain in self.chains[key]:
+            self.chains[key].remove(chain)
+            if len(self.chains[key]) == 0:
+                del self.chains[key]
+        else:
+            raise ValueError( \
+                "Chain '%s' with table '%s' with ipv '%s' not in list" % \
+                (chain, table, ipv))
 
-#    def query_chain(self, ipv, table, chain):
-#        key = (ipv, table)
-#        return (key in self.chains and chain in self.chains[key])
+    def query_chain(self, ipv, table, chain):
+        key = (ipv, table)
+        return (key in self.chains and chain in self.chains[key])
 
-#    def get_chains(self, ipv, table):
-#        key = (ipv, table)
-#        if key in self.chains:
-#            return self.chains[key]
-#        else:
-#            raise ValueError("No chains for table '%s' with ipv '%s'" % \
-#                             (table, ipv))
+    def get_chains(self, ipv, table):
+        key = (ipv, table)
+        if key in self.chains:
+            return self.chains[key]
+        else:
+            raise ValueError("No chains for table '%s' with ipv '%s'" % \
+                             (table, ipv))
 
     def get_all_chains(self):
         return self.chains
@@ -237,30 +241,38 @@ class Direct(IO_Object):
                         + "with ipv '%s' and priority %d " % (ipv, priority)
                         + "already in list, ignoring")
 
-#    def remove_rule(self, ipv, table, chain, priority, args):
-#        key = (ipv, table, chain)
-#        value = (priority, tuple(args))
-#        if key in self.rules and value in self.rules[key]:
-#            del self.rules[key][value]
-#            if len(self.rules[key]) == 0:
-#                del self.rules[key]
-#        else:
-#            raise ValueError("Rule '%s' for table '%s' and chain '%s' " % \
-#                ("',".join(args), table, chain) + \
-#                "with ipv '%s' and priority %d not in list" % (ipv, priority))
+    def remove_rule(self, ipv, table, chain, priority, args):
+        key = (ipv, table, chain)
+        value = (priority, tuple(args))
+        if key in self.rules and value in self.rules[key]:
+            del self.rules[key][value]
+            if len(self.rules[key]) == 0:
+                del self.rules[key]
+        else:
+            raise ValueError("Rule '%s' for table '%s' and chain '%s' " % \
+                ("',".join(args), table, chain) + \
+                "with ipv '%s' and priority %d not in list" % (ipv, priority))
 
-#    def query_rule(self, ipv, table, chain, priority, args):
-#        key = (ipv, table, chain)
-#        value = (priority, tuple(args))
-#        return (key in self.rules and value in self.rules[key])
+    def remove_rules(self, ipv, table, chain):
+        key = (ipv, table, chain)
+        if key in self.rules:
+            for value in self.rules[key].keys():
+                del self.rules[key][value]
+            if len(self.rules[key]) == 0:
+                del self.rules[key]
 
-#    def get_rules(self, ipv, table, chain):
-#        key = (ipv, table, chain)
-#        if key in self.rules:
-#            return self.rules[key]
-#        else:
-#            raise ValueError("No rules for table '%s' and chain '%s' " %\
-#                             (table, chain) + "with ipv '%s'" % (ipv))
+    def query_rule(self, ipv, table, chain, priority, args):
+        key = (ipv, table, chain)
+        value = (priority, tuple(args))
+        return (key in self.rules and value in self.rules[key])
+
+    def get_rules(self, ipv, table, chain):
+        key = (ipv, table, chain)
+        if key in self.rules:
+            return self.rules[key]
+        else:
+            raise ValueError("No rules for table '%s' and chain '%s' " %\
+                             (table, chain) + "with ipv '%s'" % (ipv))
 
     def get_all_rules(self):
         return self.rules
@@ -276,38 +288,40 @@ class Direct(IO_Object):
             log.warning("Passthrough '%s' for ipv '%s'" % \
                             ("',".join(args), ipv)
                         + "already in list, ignoring")
-#
-#    def remove_passthrough(self, ipv, args):
-#        if ipv in self.passthroughs and args in self.passthroughs[ipv]:
-#            self.passthroughs[ipv].remove(args)
-#            if len(self.passthroughs[ipv]) == 0:
-#                del self.passthroughs[ipv]
-#        else:
-#            raise ValueError, "Passthrough '%s' for ipv '%s'" % \
-#                ("',".join(args), ipv) + "not in list"
-#
-#    def query_passthrough(self, ipv, args):
-#        return (ipv in self.passthroughs and args in self.passthroughs[ipv])
-#
-#    def get_passthroughs(self, ipv):
-#        if ipv in self.passthroughs:
-#            return self.passthroughs[ipv]
-#        else:
-#            raise ValueError, "No passthroughs for ipv '%s'" % (ipv)
-#
+
+    def remove_passthrough(self, ipv, args):
+        if ipv in self.passthroughs and args in self.passthroughs[ipv]:
+            self.passthroughs[ipv].remove(args)
+            if len(self.passthroughs[ipv]) == 0:
+                del self.passthroughs[ipv]
+        else:
+            raise ValueError, "Passthrough '%s' for ipv '%s'" % \
+                ("',".join(args), ipv) + "not in list"
+
+    def query_passthrough(self, ipv, args):
+        return (ipv in self.passthroughs and args in self.passthroughs[ipv])
+
+    def get_passthroughs(self, ipv):
+        if ipv in self.passthroughs:
+            return self.passthroughs[ipv]
+        else:
+            raise ValueError, "No passthroughs for ipv '%s'" % (ipv)
+
     def get_all_passthroughs(self):
         return self.passthroughs
 
     # read
 
     def read(self):
-        self.clear()
+        self.cleanup()
         if not self.filename.endswith(".xml"):
             raise FirewallError(INVALID_NAME, self.filename)
         handler = direct_ContentHandler(self)
         parser = sax.make_parser()
         parser.setContentHandler(handler)
-        parser.parse(self.filename)
+        with open(self.filename, "r") as f:
+            parser.parse(f)
+
 
     def write(self):
         if os.path.exists(self.filename):
@@ -316,8 +330,8 @@ class Direct(IO_Object):
             except Exception as msg:
                 raise IOError("Backup of '%s' failed: %s" % (self.filename, msg))
 
-        fd = open(self.filename, "w")
-        handler = IO_Object_XMLGenerator(fd)
+        f = io.open(self.filename, mode='wt', encoding='UTF-8')
+        handler = IO_Object_XMLGenerator(f)
         handler.startDocument()
 
         # start whitelist element
@@ -343,7 +357,7 @@ class Direct(IO_Object):
                 handler.startElement("rule", { "ipv": ipv, "table": table,
                                                  "chain": chain,
                                                  "priority": "%d" % priority })
-                handler.ignorableWhitespace(joinArgs(args))
+                handler.ignorableWhitespace(sax.saxutils.escape(joinArgs(args)))
                 handler.endElement("rule")
                 handler.ignorableWhitespace("\n")
 
@@ -354,7 +368,7 @@ class Direct(IO_Object):
                     continue
                 handler.ignorableWhitespace("  ")
                 handler.startElement("passthrough", { "ipv": ipv })
-                handler.ignorableWhitespace(joinArgs(args))
+                handler.ignorableWhitespace(sax.saxutils.escape(joinArgs(args)))
                 handler.endElement("passthrough")
                 handler.ignorableWhitespace("\n")
 
@@ -362,4 +376,5 @@ class Direct(IO_Object):
         handler.endElement("direct")
         handler.ignorableWhitespace("\n")
         handler.endDocument()
-        fd.close()
+        f.close()
+        del handler
