@@ -19,13 +19,16 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #
 
-import os, os.path
+import os.path
+import io
 import tempfile
 import shutil
 
 from firewall.core.logger import log
+from firewall.functions import b2u, u2b, PY2
 
-valid_keys = ["DefaultZone", "MinimalMark", "CleanupOnExit", "Lockdown"]
+valid_keys = [ "DefaultZone", "MinimalMark", "CleanupOnExit", "Lockdown", 
+               "IPv6_rpfilter" ]
 
 class firewalld_conf:
     def __init__(self, filename):
@@ -36,12 +39,16 @@ class firewalld_conf:
         self._config = { }
         self._deleted = [ ]
 
+    def cleanup(self):
+        self._config.clear()
+        self._deleted = [ ]
+
     def get(self, key):
         return self._config.get(key.strip())
 
     def set(self, key, value):
-        _key = key.strip()
-        self._config[_key] = value.strip()
+        _key = b2u(key.strip())
+        self._config[_key] = b2u(value.strip())
         if _key in self._deleted:
             self._deleted.remove[_key]
 
@@ -51,7 +58,7 @@ class firewalld_conf:
             if s:
                 s += '\n'
             s += '%s=%s' % (key, value)
-        return s
+        return u2b(s) if PY2 else s
 
     # load self.filename
     def read(self):
@@ -62,7 +69,7 @@ class firewalld_conf:
             log.error("Failed to open '%s': %s" % (self.filename, msg))
             raise
 
-        for line in f.xreadlines():
+        for line in f:
             if not line:
                 break
             line = line.strip()
@@ -95,8 +102,9 @@ class firewalld_conf:
         done = [ ]
 
         try:
-            (temp_file, temp) = tempfile.mkstemp(prefix="%s." % os.path.basename(self.filename),
-                                                 dir=os.path.dirname(self.filename))
+            temp_file = tempfile.NamedTemporaryFile(mode='wt',
+                             prefix="%s." % os.path.basename(self.filename),
+                             dir=os.path.dirname(self.filename), delete=False)
         except Exception as msg:
             log.error("Failed to open temporary file: %s" % msg)
             raise
@@ -104,7 +112,7 @@ class firewalld_conf:
         modified = False
         empty = False
         try:
-            f = open(self.filename, "r")
+            f= io.open(self.filename, mode='rt', encoding='UTF-8')
         except Exception as msg:
             if os.path.exists(self.filename):
                 log.error("Failed to open '%s': %s" % (self.filename, msg))
@@ -112,7 +120,7 @@ class firewalld_conf:
             else:
                 f = None
         else:
-            for line in f.xreadlines():
+            for line in f:
                 if not line:
                     break
                 # remove newline
@@ -120,17 +128,17 @@ class firewalld_conf:
 
                 if len(line) < 1:
                     if not empty:
-                        os.write(temp_file, "\n")
+                        temp_file.write(u"\n")
                         empty = True
                 elif line[0] == '#':
                     empty = False
-                    os.write(temp_file, line)
-                    os.write(temp_file, "\n")
+                    temp_file.write(line)
+                    temp_file.write(u"\n")
                 else:
                     p = line.split("=")
                     if len(p) != 2:
                         empty = False
-                        os.write(temp_file, line+"\n")
+                        temp_file.write(line+u"\n")
                         continue
                     key = p[0].strip()
                     value = p[1].strip()
@@ -139,14 +147,14 @@ class firewalld_conf:
                         if (key in self._config and \
                                 self._config[key] != value):
                             empty = False
-                            os.write(temp_file, '%s=%s\n' \
-                                         % (key, self._config[key]))
+                            temp_file.write(u'%s=%s\n' %
+                                            (key, self._config[key]))
                             modified = True
                         elif key in self._deleted:
                             modified = True
                         else:
                             empty = False
-                            os.write(temp_file, line+"\n")
+                            temp_file.write(line+u"\n")
                         done.append(key)
                     else:
                         modified = True
@@ -157,31 +165,31 @@ class firewalld_conf:
                 if key in done:
                     continue
                 if not empty:
-                    os.write(temp_file, "\n")
+                    temp_file.write(u"\n")
                     empty = True
-                os.write(temp_file, '%s=%s\n' % (key, value))
+                temp_file.write(u'%s=%s\n' % (key, value))
                 modified = True
 
         if f:
             f.close()
-        os.close(temp_file)
+        temp_file.close()
 
         if not modified: # not modified: remove tempfile
-            os.remove(temp)
+            os.remove(temp_file.name)
             return
         # make backup
         if os.path.exists(self.filename):
             try:
                 shutil.copy2(self.filename, "%s.old" % self.filename)
             except Exception as msg:
-                os.remove(temp)
+                os.remove(temp_file.name)
                 raise IOError("Backup of '%s' failed: %s" % (self.filename, msg))
 
         # copy tempfile
         try:
-            shutil.move(temp, self.filename)
+            shutil.move(temp_file.name, self.filename)
         except Exception as msg:
-            os.remove(temp)
+            os.remove(temp_file.name)
             raise IOError("Failed to create '%s': %s" % (self.filename, msg))
         else:
-            os.chmod(self.filename, 0600)
+            os.chmod(self.filename, 0o600)
