@@ -21,12 +21,14 @@
 
 import xml.sax as sax
 import os
+import io
 import shutil
 
 from firewall.errors import *
 from firewall.core.io.io_object import *
 from firewall.core.logger import log
-from firewall.functions import uniqify
+from firewall.functions import uniqify, checkUser, checkUid, checkCommand, \
+                               checkContext, u2b_if_py2
 
 class lockdown_whitelist_ContentHandler(IO_Object_ContentHandler):
     def __init__(self, item):
@@ -34,6 +36,7 @@ class lockdown_whitelist_ContentHandler(IO_Object_ContentHandler):
         self.whitelist = False
 
     def startElement(self, name, attrs):
+        IO_Object_ContentHandler.startElement(self, name)
         self.item.parser_check_element_attrs(name, attrs)
 
         if name == "whitelist":
@@ -45,7 +48,7 @@ class lockdown_whitelist_ContentHandler(IO_Object_ContentHandler):
             if not self.whitelist:
                 log.error("Parse Error: command outside of whitelist")
                 return
-            command = str(attrs["name"])
+            command = attrs["name"]
             self.item.add_command(command)
 
         elif name == "user":
@@ -54,14 +57,14 @@ class lockdown_whitelist_ContentHandler(IO_Object_ContentHandler):
                 return
             if "id" in attrs:
                 try:
-                    uid = int(str(attrs["id"]))
+                    uid = int(attrs["id"])
                 except:
                     log.error("Parse Error: %s is not a valid uid" % 
                               attrs["id"])
                     return
                 self.item.add_uid(uid)
             elif "name" in attrs:
-                self.item.add_user(str(attrs["name"]))
+                self.item.add_user(attrs["name"])
 
         elif name == "selinux":
             if not self.whitelist:
@@ -70,7 +73,7 @@ class lockdown_whitelist_ContentHandler(IO_Object_ContentHandler):
             if not "context" in attrs:
                 log.error("Parse Error: no context")
                 return
-            self.item.add_context(str(attrs["context"]))
+            self.item.add_context(attrs["context"])
             
 
         else:
@@ -103,21 +106,52 @@ class LockdownWhitelist(IO_Object):
     def __init__(self, filename):
         super(LockdownWhitelist, self).__init__()
         self.filename = filename
-        self.clear()
-
-    def _check_config(self, config, item):
-        pass
-
-    def clear(self):
+        self.parser = None
         self.commands = [ ]
         self.contexts = [ ]
         self.users = [ ]
         self.uids = [ ]
+#        self.gids = [ ]
 #        self.groups = [ ]
 
+    def _check_config(self, config, item):
+        if item in [ "commands", "contexts", "users", "uids" ]:
+            for x in config:
+                self._check_config(x, item[:-1])
+        elif item == "command":
+            if not checkCommand(config):
+                raise FirewallError(INVALID_COMMAND, config)
+        elif item == "context":
+            if not checkContext(config):
+                raise FirewallError(INVALID_CONTEXT, config)
+        elif item == "user":
+            if not checkUser(config):
+                raise FirewallError(INVALID_USER, config)
+        elif item == "uid":
+            if not checkUid(config):
+                raise FirewallError(INVALID_UID, config)
+
+    def cleanup(self):
+        del self.commands[:]
+        del self.contexts[:]
+        del self.users[:]
+        del self.uids[:]
+#        del self.gids[:]
+#        del self.groups[:]
+
+    def encode_strings(self):
+        """ HACK. I haven't been able to make sax parser return
+            strings encoded (because of python 2) instead of in unicode.
+            Get rid of it once we throw out python 2 support."""
+        self.commands = u2b_if_py2(self.commands)
+        self.contexts = u2b_if_py2(self.contexts)
+        self.users = u2b_if_py2(self.users)
+
     # commands
 
     def add_command(self, command):
+        if not checkCommand(command):
+            raise FirewallError(INVALID_COMMAND, command)
         if command not in self.commands:
             self.commands.append(command)
 
@@ -147,6 +181,8 @@ class LockdownWhitelist(IO_Object):
     # user ids
 
     def add_uid(self, uid):
+        if not checkUid(uid):
+            raise FirewallError(INVALID_UID, str(uid))
         if uid not in self.uids:
             self.uids.append(uid)
 
@@ -169,6 +205,8 @@ class LockdownWhitelist(IO_Object):
     # users
 
     def add_user(self, user):
+        if not checkUser(user):
+            raise FirewallError(INVALID_USER, user)
         if user not in self.users:
             self.users.append(user)
 
@@ -235,6 +273,8 @@ class LockdownWhitelist(IO_Object):
     # selinux contexts
 
     def add_context(self, context):
+        if not checkContext(context):
+            raise FirewallError(INVALID_CONTEXT, context)
         if context not in self.contexts:
             self.contexts.append(context)
 
@@ -257,23 +297,27 @@ class LockdownWhitelist(IO_Object):
     # read and write
 
     def read(self):
-        self.clear()
+        self.cleanup()
         if not self.filename.endswith(".xml"):
             raise FirewallError(INVALID_NAME, self.filename)
         handler = lockdown_whitelist_ContentHandler(self)
         parser = sax.make_parser()
         parser.setContentHandler(handler)
         parser.parse(self.filename)
+        del handler
+        del parser
+        if PY2:
+            self.encode_strings()
 
     def write(self):
         if os.path.exists(self.filename):
             try:
                 shutil.copy2(self.filename, "%s.old" % self.filename)
-            except Exception, msg:
+            except Exception as msg:
                 raise IOError("Backup of '%s' failed: %s" % (self.filename, msg))
 
-        fd = open(self.filename, "w")
-        handler = IO_Object_XMLGenerator(fd)
+        f = io.open(self.filename, mode='wt', encoding='UTF-8')
+        handler = IO_Object_XMLGenerator(f)
         handler.startDocument()
 
         # start whitelist element
@@ -315,4 +359,5 @@ class LockdownWhitelist(IO_Object):
         handler.endElement("whitelist")
         handler.ignorableWhitespace("\n")
         handler.endDocument()
-        fd.close()
+        f.close()
+        del handler
