@@ -18,14 +18,14 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #
-
 import xml.sax as sax
 import os
+import io
 import shutil
 
 from firewall.config import ETC_FIREWALLD
 from firewall.errors import *
-from firewall.functions import checkIP, uniqify, max_zone_name_len
+from firewall.functions import checkIP, uniqify, max_zone_name_len, u2b_if_py2
 from firewall.core.base import DEFAULT_ZONE_TARGET, ZONE_TARGETS
 from firewall.core.io.io_object import *
 from firewall.core.rich import *
@@ -50,7 +50,7 @@ class Zone(IO_Object):
         ( "rules_str", [ "" ] ),                       # as
         )
     DBUS_SIGNATURE = '(sssbsasa(ss)asba(ssss)asasas)'
-    ADDITIONAL_ALNUM_CHARS = [ "_" ]
+    ADDITIONAL_ALNUM_CHARS = [ "_", "/" ]
     PARSER_REQUIRED_ELEMENT_ATTRS = {
         "short": None,
         "description": None,
@@ -106,6 +106,41 @@ class Zone(IO_Object):
         self.fw_config = None # to be able to check services and a icmp_blocks
         self.rules = [ ]
         self.combined = False
+        self.applied = False
+
+    def cleanup(self):
+        self.version = ""
+        self.short = ""
+        self.description = ""
+        self.UNUSED = False
+        self.target = DEFAULT_ZONE_TARGET
+        del self.services[:]
+        del self.ports[:]
+        del self.icmp_blocks[:]
+        self.masquerade = False
+        del self.forward_ports[:]
+        del self.interfaces[:]
+        del self.sources[:]
+        self.fw_config = None # to be able to check services and a icmp_blocks
+        del self.rules[:]
+        self.combined = False
+        self.applied = False
+
+    def encode_strings(self):
+        """ HACK. I haven't been able to make sax parser return
+            strings encoded (because of python 2) instead of in unicode.
+            Get rid of it once we throw out python 2 support."""
+        self.version = u2b_if_py2(self.version)
+        self.short = u2b_if_py2(self.short)
+        self.description = u2b_if_py2(self.description)
+        self.target = u2b_if_py2(self.target)
+        self.services = [u2b_if_py2(s) for s in self.services]
+        self.ports = [(u2b_if_py2(po),u2b_if_py2(pr)) for (po,pr) in self.ports]
+        self.icmp_blocks = [u2b_if_py2(i) for i in self.icmp_blocks]
+        self.forward_ports = [(u2b_if_py2(p1),u2b_if_py2(p2),u2b_if_py2(p3),u2b_if_py2(p4)) for (p1,p2,p3,p4) in self.forward_ports]
+        self.interfaces = [u2b_if_py2(i) for i in self.interfaces]
+        self.sources = [u2b_if_py2(s) for s in self.sources]
+        self.rules = [u2b_if_py2(s) for s in self.rules]
 
     def __getattr__(self, name):
         if name == "rules_str":
@@ -146,6 +181,14 @@ class Zone(IO_Object):
                 if fwd_port[3]:
                     if not checkIP(fwd_port[3]):
                         raise FirewallError(INVALID_ADDR, fwd_port[3])
+        elif item == "target":
+            if config not in ZONE_TARGETS:
+                raise FirewallError(INVALID_TARGET, config)
+
+    def check_name(self, name):
+        super(Zone, self).check_name(name)
+        if name[0] == '/' or name [-1] == '/' or name.count('/') > 1:
+            raise FirewallError(INVALID_NAME, name)
 
     def combine(self, zone):
         self.combined = True
@@ -187,6 +230,7 @@ class zone_ContentHandler(IO_Object_ContentHandler):
         self._limit_ok = None
 
     def startElement(self, name, attrs):
+        IO_Object_ContentHandler.startElement(self, name)
         if self._rule_error:
             return
 
@@ -197,12 +241,12 @@ class zone_ContentHandler(IO_Object_ContentHandler):
                 log.warning("Ignoring deprecated attribute name='%s'" % 
                             attrs["name"])
             if "version" in attrs:
-                self.item.version = str(attrs["version"])
+                self.item.version = attrs["version"]
             if "immutable" in attrs:
                 log.warning("Ignoring deprecated attribute immutable='%s'" % 
                             attrs["immutable"])
             if "target" in attrs:
-                target = str(attrs["target"])
+                target = attrs["target"]
                 if target not in ZONE_TARGETS:
                     raise FirewallError(INVALID_TARGET, target)
                 if target != "" and target != DEFAULT_ZONE_TARGET:
@@ -218,21 +262,21 @@ class zone_ContentHandler(IO_Object_ContentHandler):
                     log.error('Invalid rule: More than one element, ignoring.')
                     self._rule_error = True
                     return
-                self._rule.element = Rich_Service(str(attrs["name"]))
+                self._rule.element = Rich_Service(attrs["name"])
                 return
-            if str(attrs["name"]) not in self.item.services:
-                self.item.services.append(str(attrs["name"]))
+            if attrs["name"] not in self.item.services:
+                self.item.services.append(attrs["name"])
         elif name == "port":
             if self._rule:
                 if self._rule.element:
                     log.error('Invalid rule: More than one element, ignoring.')
                     self._rule_error = True
                     return
-                self._rule.element = Rich_Port(str(attrs["port"]),
-                                               str(attrs["protocol"]))
+                self._rule.element = Rich_Port(attrs["port"],
+                                               attrs["protocol"])
                 return
             # TODO: fix port string according to fw_zone.__port_id()
-            entry = (str(attrs["port"]), str(attrs["protocol"]))
+            entry = (attrs["port"], attrs["protocol"])
             if entry not in self.item.ports:
                 self.item.ports.append(entry)
         elif name == "protocol":
@@ -241,7 +285,7 @@ class zone_ContentHandler(IO_Object_ContentHandler):
                     log.error('Invalid rule: More than one element, ignoring.')
                     self._rule_error = True
                     return
-                self._rule.element = Rich_Protocol(str(attrs["value"]))
+                self._rule.element = Rich_Protocol(attrs["value"])
             else:
                 log.error('Protocol allowed only in rule.')
         elif name == "icmp-block":
@@ -250,41 +294,41 @@ class zone_ContentHandler(IO_Object_ContentHandler):
                     log.error('Invalid rule: More than one element, ignoring.')
                     self._rule_error = True
                     return
-                self._rule.element = Rich_IcmpBlock(str(attrs["name"]))
+                self._rule.element = Rich_IcmpBlock(attrs["name"])
                 return
-            if str(attrs["name"]) not in self.item.icmp_blocks:
-                self.item.icmp_blocks.append(str(attrs["name"]))
+            if attrs["name"] not in self.item.icmp_blocks:
+                self.item.icmp_blocks.append(attrs["name"])
         elif name == "masquerade":
+            if "enabled" in attrs:
+                log.warning("Ignoring deprecated attribute enabled='%s'" %
+                            attrs["enabled"])
             if self._rule:
-                if "enabled" in attrs:
-                    log.warning('Invalid rule: Masquerade attribute ignored in rule.')
                 if self._rule.element:
                     log.error('Invalid rule: More than one element, ignoring.')
                     self._rule_error = True
                     return
                 self._rule.element = Rich_Masquerade()
-                return
-            if attrs["enabled"].lower() in [ "yes", "true" ]:
+            else:
                 self.item.masquerade = True
         elif name == "forward-port":
             to_port = ""
             if "to-port" in attrs:
-                to_port = str(attrs["to-port"])
+                to_port = attrs["to-port"]
             to_addr = ""
             if "to-addr" in attrs:
-                to_addr = str(attrs["to-addr"])
+                to_addr = attrs["to-addr"]
 
             if self._rule:
                 if self._rule.element:
                     log.error('Invalid rule: More than one element, ignoring.')
                     self._rule_error = True
                     return
-                self._rule.element = Rich_ForwardPort(str(attrs["port"]),
-                                                      str(attrs["protocol"]),
+                self._rule.element = Rich_ForwardPort(attrs["port"],
+                                                      attrs["protocol"],
                                                       to_port, to_addr)
                 return
             # TODO: fix port string according to fw_zone.__forward_port_id()
-            entry = (str(attrs["port"]), str(attrs["protocol"]), to_port,
+            entry = (attrs["port"], attrs["protocol"], to_port,
                      to_addr)
             if entry not in self.item.forward_ports:
                 self.item.forward_ports.append(entry)
@@ -299,7 +343,7 @@ class zone_ContentHandler(IO_Object_ContentHandler):
                 log.error('Invalid interface: Name missing.')
                 self._rule_error = True
                 return
-            name = str(attrs["name"])
+            name = attrs["name"]
             if name not in self.item.interfaces:
                 self.item.interfaces.append(name)
             
@@ -313,7 +357,7 @@ class zone_ContentHandler(IO_Object_ContentHandler):
                 if "invert" in attrs and \
                         attrs["invert"].lower() in [ "yes", "true" ]:
                     invert = True
-                self._rule.source = Rich_Source(str(attrs["address"]), invert)
+                self._rule.source = Rich_Source(attrs["address"], invert)
                 return
             # zone bound to source
             if not "address" in attrs:
@@ -325,7 +369,7 @@ class zone_ContentHandler(IO_Object_ContentHandler):
             if "invert" in attrs:
                 log.error('Invalid source: Invertion not allowed here.')
                 return
-            entry = str(attrs["address"])
+            entry = attrs["address"]
             if entry not in self.item.sources:
                 self.item.sources.append(entry)
 
@@ -341,7 +385,7 @@ class zone_ContentHandler(IO_Object_ContentHandler):
             if "invert" in attrs and \
                     attrs["invert"].lower() in [ "yes", "true" ]:
                 invert = True
-            self._rule.destination = Rich_Destination(str(attrs["address"]),
+            self._rule.destination = Rich_Destination(attrs["address"],
                                                       invert)
 
         elif name in [ "accept", "reject", "drop" ]:
@@ -358,7 +402,7 @@ class zone_ContentHandler(IO_Object_ContentHandler):
             if name == "reject":
                 _type = None
                 if "type" in attrs:
-                    _type = str(attrs["type"])
+                    _type = attrs["type"]
                 self._rule.action = Rich_Reject(_type)
             if name == "drop":
                 self._rule.action = Rich_Drop()
@@ -373,13 +417,13 @@ class zone_ContentHandler(IO_Object_ContentHandler):
                 return
             level = None
             if "level" in attrs:
-                level = str(attrs["level"])
+                level = attrs["level"]
                 if level not in [ "emerg", "alert", "crit", "error",
                                   "warning", "notice", "info", "debug" ]:
                     log.error('Invalid rule: Invalid log level')
                     self._rule_error = True
                     return
-            prefix = str(attrs["prefix"]) if "prefix" in attrs else None
+            prefix = attrs["prefix"] if "prefix" in attrs else None
             self._rule.log = Rich_Log(prefix, level)
             self._limit_ok = self._rule.log
 
@@ -415,7 +459,7 @@ class zone_ContentHandler(IO_Object_ContentHandler):
                 log.error('Invalid rule: More than one limit')
                 self._rule_error = True
                 return
-            value = str(attrs["value"])
+            value = attrs["value"]
             self._limit_ok.limit = Rich_Limit(value)
 
         else:
@@ -456,6 +500,10 @@ def zone_reader(filename, path):
     name = "%s/%s" % (path, filename)
     with open(name, "r") as f:
         parser.parse(f)
+    del handler
+    del parser
+    if PY2:
+        zone.encode_strings()
     return zone
 
 def zone_writer(zone, path=None):
@@ -469,11 +517,15 @@ def zone_writer(zone, path=None):
     if os.path.exists(name):
         try:
             shutil.copy2(name, "%s.old" % name)
-        except Exception, msg:
+        except Exception as msg:
             raise IOError("Backup of '%s' failed: %s" % (name, msg))
 
-    fd = open(name, "w")
-    handler = IO_Object_XMLGenerator(fd)
+    dirpath = os.path.dirname(name)
+    if dirpath.startswith(ETC_FIREWALLD) and not os.path.exists(dirpath):
+        os.mkdir(dirpath, 0o750)
+
+    f = io.open(name, mode='wt', encoding='UTF-8')
+    handler = IO_Object_XMLGenerator(f)
     handler.startDocument()
 
     # start zone element
@@ -489,9 +541,7 @@ def zone_writer(zone, path=None):
     if zone.short and zone.short != "":
         handler.ignorableWhitespace("  ")
         handler.startElement("short", { })
-        handler.characters(zone.short.decode('utf-8')
-                           if isinstance(zone.short, bytes)
-                           else zone.short)
+        handler.characters(zone.short)
         handler.endElement("short")
         handler.ignorableWhitespace("\n")
 
@@ -499,9 +549,7 @@ def zone_writer(zone, path=None):
     if zone.description and zone.description != "":
         handler.ignorableWhitespace("  ")
         handler.startElement("description", { })
-        handler.characters(zone.description.decode('utf-8')
-                           if isinstance(zone.description, bytes)
-                           else zone.description)
+        handler.characters(zone.description)
         handler.endElement("description")
         handler.ignorableWhitespace("\n")
 
@@ -538,7 +586,7 @@ def zone_writer(zone, path=None):
     # masquerade
     if zone.masquerade:
         handler.ignorableWhitespace("  ")
-        handler.simpleElement("masquerade", { "enabled": "Yes" })
+        handler.simpleElement("masquerade", { })
         handler.ignorableWhitespace("\n")
 
     # forward-ports
@@ -638,6 +686,7 @@ def zone_writer(zone, path=None):
 
         # audit
         if rule.audit:
+            attrs = {}
             if rule.audit.limit:
                 handler.ignorableWhitespace("    ")
                 handler.startElement("audit", { })
@@ -686,4 +735,5 @@ def zone_writer(zone, path=None):
     handler.endElement("zone")
     handler.ignorableWhitespace("\n")
     handler.endDocument()
-    fd.close()
+    f.close()
+    del handler
