@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2010-2012 Red Hat, Inc.
+# Copyright (C) 2010-2014 Red Hat, Inc.
 #
 # Authors:
 # Thomas Woerner <twoerner@redhat.com>
@@ -38,6 +38,7 @@ from firewall.server.decorators import *
 from firewall.server.config import FirewallDConfig
 from firewall.dbus_utils import dbus_to_python, \
     command_of_sender, context_of_sender, uid_of_sender, user_of_uid
+from firewall.core.io.zone import Zone
 from firewall.core.io.service import Service
 from firewall.core.io.icmptype import IcmpType
 from firewall.errors import *
@@ -147,6 +148,9 @@ class FirewallD(slip.dbus.service.Object):
         elif prop == "IPv6":
             return self.fw.ip6tables_enabled
 
+        elif prop == "IPv6_rpfilter":
+            return self.fw.ipv6_rpfilter_enabled
+
         elif prop == "BRIDGE":
             return self.fw.ebtables_enabled
 
@@ -189,6 +193,7 @@ class FirewallD(slip.dbus.service.Object):
             'state': self._get_property("state"),
             'IPv4': self._get_property("IPv4"),
             'IPv6': self._get_property("IPv6"),
+            'IPv6_rpfilter': self._get_property("IPv6_rpfilter"),
             'BRIDGE': self._get_property("BRIDGE"),
         }
         
@@ -246,6 +251,7 @@ class FirewallD(slip.dbus.service.Object):
         log.debug1("completeReload()")
 
         self.fw.reload(True)
+        self.config.reload()
         self.Reloaded()
 
     @dbus.service.signal(DBUS_INTERFACE)
@@ -283,7 +289,7 @@ class FirewallD(slip.dbus.service.Object):
         self.fw.policies.disable_lockdown()
         self.LockdownDisabled()
 
-    @slip.dbus.polkit.require_auth(PK_ACTION_POLICIES)
+    @slip.dbus.polkit.require_auth(PK_ACTION_POLICIES_INFO)
     @dbus_service_method(DBUS_INTERFACE_POLICIES, in_signature='',
                          out_signature='b')
     @dbus_handle_exceptions
@@ -334,7 +340,7 @@ class FirewallD(slip.dbus.service.Object):
         self.fw.policies.lockdown_whitelist.remove_command(command)
         self.LockdownWhitelistCommandRemoved(command)
 
-    @slip.dbus.polkit.require_auth(PK_ACTION_POLICIES)
+    @slip.dbus.polkit.require_auth(PK_ACTION_POLICIES_INFO)
     @dbus_service_method(DBUS_INTERFACE_POLICIES, in_signature='s',
                          out_signature='b')
     @dbus_handle_exceptions
@@ -346,7 +352,7 @@ class FirewallD(slip.dbus.service.Object):
         # no access check here
         return self.fw.policies.lockdown_whitelist.has_command(command)
 
-    @slip.dbus.polkit.require_auth(PK_ACTION_POLICIES)
+    @slip.dbus.polkit.require_auth(PK_ACTION_POLICIES_INFO)
     @dbus_service_method(DBUS_INTERFACE_POLICIES, in_signature='',
                          out_signature='as')
     @dbus_handle_exceptions
@@ -395,7 +401,7 @@ class FirewallD(slip.dbus.service.Object):
         self.fw.policies.lockdown_whitelist.remove_uid(uid)
         self.LockdownWhitelistUidRemoved(uid)
 
-    @slip.dbus.polkit.require_auth(PK_ACTION_POLICIES)
+    @slip.dbus.polkit.require_auth(PK_ACTION_POLICIES_INFO)
     @dbus_service_method(DBUS_INTERFACE_POLICIES, in_signature='i',
                          out_signature='b')
     @dbus_handle_exceptions
@@ -407,7 +413,7 @@ class FirewallD(slip.dbus.service.Object):
         # no access check here
         return self.fw.policies.lockdown_whitelist.has_uid(uid)
 
-    @slip.dbus.polkit.require_auth(PK_ACTION_POLICIES)
+    @slip.dbus.polkit.require_auth(PK_ACTION_POLICIES_INFO)
     @dbus_service_method(DBUS_INTERFACE_POLICIES, in_signature='',
                          out_signature='ai')
     @dbus_handle_exceptions
@@ -456,7 +462,7 @@ class FirewallD(slip.dbus.service.Object):
         self.fw.policies.lockdown_whitelist.remove_user(user)
         self.LockdownWhitelistUserRemoved(user)
 
-    @slip.dbus.polkit.require_auth(PK_ACTION_POLICIES)
+    @slip.dbus.polkit.require_auth(PK_ACTION_POLICIES_INFO)
     @dbus_service_method(DBUS_INTERFACE_POLICIES, in_signature='s',
                          out_signature='b')
     @dbus_handle_exceptions
@@ -468,7 +474,7 @@ class FirewallD(slip.dbus.service.Object):
         # no access check here
         return self.fw.policies.lockdown_whitelist.has_user(user)
 
-    @slip.dbus.polkit.require_auth(PK_ACTION_POLICIES)
+    @slip.dbus.polkit.require_auth(PK_ACTION_POLICIES_INFO)
     @dbus_service_method(DBUS_INTERFACE_POLICIES, in_signature='',
                          out_signature='as')
     @dbus_handle_exceptions
@@ -517,7 +523,7 @@ class FirewallD(slip.dbus.service.Object):
         self.fw.policies.lockdown_whitelist.remove_context(context)
         self.LockdownWhitelistContextRemoved(context)
 
-    @slip.dbus.polkit.require_auth(PK_ACTION_POLICIES)
+    @slip.dbus.polkit.require_auth(PK_ACTION_POLICIES_INFO)
     @dbus_service_method(DBUS_INTERFACE_POLICIES, in_signature='s',
                          out_signature='b')
     @dbus_handle_exceptions
@@ -529,7 +535,7 @@ class FirewallD(slip.dbus.service.Object):
         # no access check here
         return self.fw.policies.lockdown_whitelist.has_context(context)
 
-    @slip.dbus.polkit.require_auth(PK_ACTION_POLICIES)
+    @slip.dbus.polkit.require_auth(PK_ACTION_POLICIES_INFO)
     @dbus_service_method(DBUS_INTERFACE_POLICIES, in_signature='',
                          out_signature='as')
     @dbus_handle_exceptions
@@ -554,7 +560,7 @@ class FirewallD(slip.dbus.service.Object):
 
     # PANIC
 
-    @slip.dbus.polkit.require_auth(PK_ACTION_INFO)
+    @slip.dbus.polkit.require_auth(PK_ACTION_CONFIG)
     @dbus_service_method(DBUS_INTERFACE, in_signature='', out_signature='')
     @dbus_handle_exceptions
     def enablePanicMode(self, sender=None):
@@ -567,7 +573,7 @@ class FirewallD(slip.dbus.service.Object):
         self.fw.enable_panic_mode()
         self.PanicModeEnabled()
 
-    @slip.dbus.polkit.require_auth(PK_ACTION_INFO)
+    @slip.dbus.polkit.require_auth(PK_ACTION_CONFIG)
     @dbus_service_method(DBUS_INTERFACE, in_signature='', out_signature='')
     @dbus_handle_exceptions
     def disablePanicMode(self, sender=None):
@@ -603,6 +609,16 @@ class FirewallD(slip.dbus.service.Object):
 
     # list functions
 
+    @slip.dbus.polkit.require_auth(PK_ACTION_INFO)
+    @dbus_service_method(DBUS_INTERFACE, in_signature='s',
+                         out_signature=Zone.DBUS_SIGNATURE)
+    @dbus_handle_exceptions
+    def getZoneSettings(self, zone, sender=None):
+        # returns zone settings for zone
+        zone = dbus_to_python(zone)
+        log.debug1("getZoneSettings(%s)", zone)
+        return self.fw.zone.get_zone(zone).export_config()
+
     @slip.dbus.polkit.require_auth(PK_ACTION_INFO)
     @dbus_service_method(DBUS_INTERFACE, in_signature='',
                          out_signature='as')
@@ -739,7 +755,7 @@ class FirewallD(slip.dbus.service.Object):
             return zone
         return ""
 
-    @slip.dbus.polkit.require_auth(PK_ACTION_CONFIG)
+    @slip.dbus.polkit.require_auth(PK_ACTION_CONFIG_INFO)
     @dbus_service_method(DBUS_INTERFACE_ZONE, in_signature='s',
                          out_signature='b')
     @dbus_handle_exceptions
@@ -816,7 +832,7 @@ class FirewallD(slip.dbus.service.Object):
         self.InterfaceRemoved(_zone, interface)
         return _zone
 
-    @slip.dbus.polkit.require_auth(PK_ACTION_CONFIG)
+    @slip.dbus.polkit.require_auth(PK_ACTION_CONFIG_INFO)
     @dbus_service_method(DBUS_INTERFACE_ZONE, in_signature='ss',
                          out_signature='b')
     @dbus_handle_exceptions
@@ -829,7 +845,7 @@ class FirewallD(slip.dbus.service.Object):
         log.debug1("zone.queryInterface('%s', '%s')" % (zone, interface))
         return self.fw.zone.query_interface(zone, interface)
 
-    @slip.dbus.polkit.require_auth(PK_ACTION_CONFIG)
+    @slip.dbus.polkit.require_auth(PK_ACTION_CONFIG_INFO)
     @dbus_service_method(DBUS_INTERFACE_ZONE, in_signature='s',
                          out_signature='as')
     @dbus_handle_exceptions
@@ -923,7 +939,7 @@ class FirewallD(slip.dbus.service.Object):
         self.SourceRemoved(_zone, source)
         return _zone
 
-    @slip.dbus.polkit.require_auth(PK_ACTION_CONFIG)
+    @slip.dbus.polkit.require_auth(PK_ACTION_CONFIG_INFO)
     @dbus_service_method(DBUS_INTERFACE_ZONE, in_signature='ss',
                          out_signature='b')
     @dbus_handle_exceptions
@@ -936,7 +952,7 @@ class FirewallD(slip.dbus.service.Object):
         log.debug1("zone.querySource('%s', '%s')" % (zone, source))
         return self.fw.zone.query_source(zone, source)
 
-    @slip.dbus.polkit.require_auth(PK_ACTION_CONFIG)
+    @slip.dbus.polkit.require_auth(PK_ACTION_CONFIG_INFO)
     @dbus_service_method(DBUS_INTERFACE_ZONE, in_signature='s',
                          out_signature='as')
     @dbus_handle_exceptions
@@ -1013,7 +1029,7 @@ class FirewallD(slip.dbus.service.Object):
         self.RichRuleRemoved(_zone, rule)
         return _zone
 
-    @slip.dbus.polkit.require_auth(PK_ACTION_CONFIG)
+    @slip.dbus.polkit.require_auth(PK_ACTION_CONFIG_INFO)
     @dbus_service_method(DBUS_INTERFACE_ZONE, in_signature='ss',
                          out_signature='b')
     @dbus_handle_exceptions
@@ -1025,12 +1041,12 @@ class FirewallD(slip.dbus.service.Object):
         obj = Rich_Rule(rule_str=rule)
         return self.fw.zone.query_rule(zone, obj)
 
-    @slip.dbus.polkit.require_auth(PK_ACTION_CONFIG)
+    @slip.dbus.polkit.require_auth(PK_ACTION_CONFIG_INFO)
     @dbus_service_method(DBUS_INTERFACE_ZONE, in_signature='s',
                          out_signature='as')
     @dbus_handle_exceptions
     def getRichRules(self, zone, sender=None):
-        # returns the list of enabled services for zone
+        # returns the list of enabled rich rules for zone
         # TODO: should be renamed to listRichRules()
         # because is called by firewall-cmd --zone --list-rich-rules
         zone = dbus_to_python(zone)
@@ -1097,7 +1113,7 @@ class FirewallD(slip.dbus.service.Object):
         self.ServiceRemoved(_zone, service)
         return _zone
 
-    @slip.dbus.polkit.require_auth(PK_ACTION_CONFIG)
+    @slip.dbus.polkit.require_auth(PK_ACTION_CONFIG_INFO)
     @dbus_service_method(DBUS_INTERFACE_ZONE, in_signature='ss',
                          out_signature='b')
     @dbus_handle_exceptions
@@ -1108,7 +1124,7 @@ class FirewallD(slip.dbus.service.Object):
         log.debug1("zone.queryService('%s', '%s')" % (zone, service))
         return self.fw.zone.query_service(zone, service)
 
-    @slip.dbus.polkit.require_auth(PK_ACTION_CONFIG)
+    @slip.dbus.polkit.require_auth(PK_ACTION_CONFIG_INFO)
     @dbus_service_method(DBUS_INTERFACE_ZONE, in_signature='s',
                          out_signature='as')
     @dbus_handle_exceptions
@@ -1184,7 +1200,7 @@ class FirewallD(slip.dbus.service.Object):
         self.PortRemoved(_zone, port, protocol)
         return _zone
 
-    @slip.dbus.polkit.require_auth(PK_ACTION_CONFIG)
+    @slip.dbus.polkit.require_auth(PK_ACTION_CONFIG_INFO)
     @dbus_service_method(DBUS_INTERFACE_ZONE, in_signature='sss',
                          out_signature='b')
     @dbus_handle_exceptions
@@ -1196,7 +1212,7 @@ class FirewallD(slip.dbus.service.Object):
         log.debug1("zone.queryPort('%s', '%s', '%s')" % (zone, port, protocol))
         return self.fw.zone.query_port(zone, port, protocol)
 
-    @slip.dbus.polkit.require_auth(PK_ACTION_CONFIG)
+    @slip.dbus.polkit.require_auth(PK_ACTION_CONFIG_INFO)
     @dbus_service_method(DBUS_INTERFACE_ZONE, in_signature='s',
                          out_signature='aas')
     @dbus_handle_exceptions
@@ -1265,7 +1281,7 @@ class FirewallD(slip.dbus.service.Object):
         self.MasqueradeRemoved(_zone)
         return _zone
 
-    @slip.dbus.polkit.require_auth(PK_ACTION_CONFIG)
+    @slip.dbus.polkit.require_auth(PK_ACTION_CONFIG_INFO)
     @dbus_service_method(DBUS_INTERFACE_ZONE, in_signature='s',
                          out_signature='b')
     @dbus_handle_exceptions
@@ -1346,7 +1362,7 @@ class FirewallD(slip.dbus.service.Object):
         self.ForwardPortRemoved(_zone, port, protocol, toport, toaddr)
         return _zone
 
-    @slip.dbus.polkit.require_auth(PK_ACTION_CONFIG)
+    @slip.dbus.polkit.require_auth(PK_ACTION_CONFIG_INFO)
     @dbus_service_method(DBUS_INTERFACE_ZONE, in_signature='sssss',
                          out_signature='b')
     @dbus_handle_exceptions
@@ -1363,7 +1379,7 @@ class FirewallD(slip.dbus.service.Object):
         return self.fw.zone.query_forward_port(zone, port, protocol, toport,
                                                toaddr)
 
-    @slip.dbus.polkit.require_auth(PK_ACTION_CONFIG)
+    @slip.dbus.polkit.require_auth(PK_ACTION_CONFIG_INFO)
     @dbus_service_method(DBUS_INTERFACE_ZONE, in_signature='s',
                          out_signature='aas')
     @dbus_handle_exceptions
@@ -1436,7 +1452,7 @@ class FirewallD(slip.dbus.service.Object):
         self.IcmpBlockRemoved(_zone, icmp)
         return _zone
 
-    @slip.dbus.polkit.require_auth(PK_ACTION_CONFIG)
+    @slip.dbus.polkit.require_auth(PK_ACTION_CONFIG_INFO)
     @dbus_service_method(DBUS_INTERFACE_ZONE, in_signature='ss',
                          out_signature='b')
     @dbus_handle_exceptions
@@ -1447,14 +1463,14 @@ class FirewallD(slip.dbus.service.Object):
         log.debug1("zone.queryIcmpBlock('%s', '%s')" % (zone, icmp))
         return self.fw.zone.query_icmp_block(zone, icmp)
 
-    @slip.dbus.polkit.require_auth(PK_ACTION_CONFIG)
+    @slip.dbus.polkit.require_auth(PK_ACTION_CONFIG_INFO)
     @dbus_service_method(DBUS_INTERFACE_ZONE, in_signature='s',
                          out_signature='as')
     @dbus_handle_exceptions
     def getIcmpBlocks(self, zone, sender=None):
         # returns the list of enabled icmpblocks
         # TODO: should be renamed to listIcmpBlocks()
-        # because is called by firewall-cmd --zone --list-icmptypes
+        # because is called by firewall-cmd --zone --list-icmp-blocks
         zone = dbus_to_python(zone)
         log.debug1("zone.getIcmpBlocks('%s')" % (zone))
         return self.fw.zone.list_icmp_blocks(zone)
@@ -1504,7 +1520,7 @@ class FirewallD(slip.dbus.service.Object):
         self.fw.direct.remove_chain(ipv, table, chain)
         self.ChainRemoved(ipv, table, chain)
     
-    @slip.dbus.polkit.require_auth(PK_ACTION_DIRECT)
+    @slip.dbus.polkit.require_auth(PK_ACTION_DIRECT_INFO)
     @dbus_service_method(DBUS_INTERFACE_DIRECT, in_signature='sss',
                          out_signature='b')
     @dbus_handle_exceptions
@@ -1516,7 +1532,7 @@ class FirewallD(slip.dbus.service.Object):
         log.debug1("direct.queryChain('%s', '%s', '%s')" % (ipv, table, chain))
         return self.fw.direct.query_chain(ipv, table, chain)
 
-    @slip.dbus.polkit.require_auth(PK_ACTION_DIRECT)
+    @slip.dbus.polkit.require_auth(PK_ACTION_DIRECT_INFO)
     @dbus_service_method(DBUS_INTERFACE_DIRECT, in_signature='ss',
                          out_signature='as')
     @dbus_handle_exceptions
@@ -1527,7 +1543,7 @@ class FirewallD(slip.dbus.service.Object):
         log.debug1("direct.getChains('%s', '%s')" % (ipv, table))
         return self.fw.direct.get_chains(ipv, table)
 
-    @slip.dbus.polkit.require_auth(PK_ACTION_DIRECT)
+    @slip.dbus.polkit.require_auth(PK_ACTION_DIRECT_INFO)
     @dbus_service_method(DBUS_INTERFACE_DIRECT, in_signature='',
                          out_signature='a(sss)')
     @dbus_handle_exceptions
@@ -1585,6 +1601,21 @@ class FirewallD(slip.dbus.service.Object):
         self.RuleRemoved(ipv, table, chain, priority, args)
     
     @slip.dbus.polkit.require_auth(PK_ACTION_DIRECT)
+    @dbus_service_method(DBUS_INTERFACE_DIRECT, in_signature='sss',
+                         out_signature='')
+    @dbus_handle_exceptions
+    def removeRules(self, ipv, table, chain, sender=None):
+        # removes direct rule
+        ipv = dbus_to_python(ipv)
+        table = dbus_to_python(table)
+        chain = dbus_to_python(chain)
+        log.debug1("direct.removeRules('%s', '%s', '%s')" % (ipv, table, chain))
+        self.accessCheck(sender)
+        for (priority, args) in self.fw.direct.get_rules(ipv, table, chain):
+            self.fw.direct.remove_rule(ipv, table, chain, priority, args)
+            self.RuleRemoved(ipv, table, chain, priority, args)
+
+    @slip.dbus.polkit.require_auth(PK_ACTION_DIRECT_INFO)
     @dbus_service_method(DBUS_INTERFACE_DIRECT, in_signature='sssias',
                          out_signature='b')
     @dbus_handle_exceptions
@@ -1599,7 +1630,7 @@ class FirewallD(slip.dbus.service.Object):
                        (table, chain, priority, "','".join(args)))
         return self.fw.direct.query_rule(ipv, table, chain, priority, args)
 
-    @slip.dbus.polkit.require_auth(PK_ACTION_DIRECT)
+    @slip.dbus.polkit.require_auth(PK_ACTION_DIRECT_INFO)
     @dbus_service_method(DBUS_INTERFACE_DIRECT, in_signature='sss',
                          out_signature='a(ias)')
     @dbus_handle_exceptions
@@ -1611,7 +1642,7 @@ class FirewallD(slip.dbus.service.Object):
         log.debug1("direct.getRules('%s', '%s', '%s')" % (ipv, table, chain))
         return self.fw.direct.get_rules(ipv, table, chain)
 
-    @slip.dbus.polkit.require_auth(PK_ACTION_DIRECT)
+    @slip.dbus.polkit.require_auth(PK_ACTION_DIRECT_INFO)
     @dbus_service_method(DBUS_INTERFACE_DIRECT, in_signature='',
                          out_signature='a(sssias)')
     @dbus_handle_exceptions
@@ -1647,3 +1678,15 @@ class FirewallD(slip.dbus.service.Object):
         log.debug1("direct.passthrough('%s', '%s')" % (ipv, "','".join(args)))
         self.accessCheck(sender)
         return self.fw.direct.passthrough(ipv, args)
+
+    # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
+
+    @slip.dbus.polkit.require_auth(PK_ACTION_ALL)
+    @dbus_service_method(DBUS_INTERFACE, in_signature='', out_signature='')
+    @dbus_handle_exceptions
+    def authorizeAll(self, sender=None):
+        """ PK_ACTION_ALL implies all other actions, i.e. once a subject is
+            authorized for PK_ACTION_ALL it's also authorized for any other action.
+            Use-case is GUI (RHBZ#994729).
+        """
+        pass
